package main

import (
	"fmt"
	"log"
	"os"

	"git.pbiernat.dev/egommerce/catalog-service/internal/app/config"
	"git.pbiernat.dev/egommerce/catalog-service/internal/app/database"
	"git.pbiernat.dev/egommerce/catalog-service/internal/app/server"
	"git.pbiernat.dev/egommerce/go-api-pkg/fluentd"
	amqp "git.pbiernat.dev/egommerce/go-api-pkg/rabbitmq"
	"github.com/google/uuid"
)

const (
	defAppName       = "catalog-worker"
	defLoggerAddr    = "api-logger:24224"
	defDbURL         = "postgres://postgres:12345678@postgres-db:5432/egommerce"
	defMongoDbURL    = "mongodb://mongodb:12345678@mongo-db:27017"
	defEventBusURL   = "amqp://guest:guest@api-eventbus:5672"
	ebEventsExchange = "api-events"
	ebEventsQueue    = "catalog-worker"
)

func main() {
	id := uuid.New().String()[24:]
	if config.ErrLoadingEnvs != nil {
		log.Panicln("Error loading .env file", config.ErrLoadingEnvs)
	}

	c := new(server.Config)
	c.AppName = config.GetEnv("APP_NAME", defAppName) + "#:" + id
	c.LoggerAddr = config.GetEnv("LOGGER_ADDR", defLoggerAddr)
	c.DbURL = config.GetEnv("DATABASE_URL", defDbURL)
	c.EventBusURL = config.GetEnv("EVENTBUS_URL", defEventBusURL)
	c.EventBusExchange = ebEventsExchange
	c.EventBusQueue = ebEventsQueue
	// c.EventBusQueue = fmt.Sprintf("%s-%s", ebEventsQueue, id)

	logHost, logPort := fluentd.ParseAddr(c.LoggerAddr)
	logger := fluentd.NewLogger(c.AppName, logHost, logPort)
	defer logger.Close()

	// db conn
	dbConn, err := database.Connect(c.DbURL)
	if err != nil { // fixme: add wait-for-db...
		logger.Log("Failed to connect to Database server: %v\n", err)
		os.Exit(1)
	}
	defer dbConn.Close()

	// eventbus conn
	ebConn, ebCh, err := amqp.Open(c.EventBusURL)
	if err != nil {
		logger.Log("Failed to connect to EventBus server: %v\n", err)
		os.Exit(1)
	}
	defer ebCh.Close()
	defer amqp.Close(ebConn)

	err = amqp.NewExchange(ebCh, c.EventBusExchange)
	if err != nil {
		logger.Log("Failed to declare EventBus exchange: %v\n", err)
		os.Exit(1)
	}

	// create and bind queues
	_, err = ebCh.QueueDeclare(
		c.EventBusQueue, // name
		false,           // durable
		false,           // delete when unused
		false,           // exclusive
		false,           // no-wait
		nil,             // arguments
	)
	if err != nil {
		logger.Log("Failed to declare EventBus queue: %v\n", err)
		os.Exit(1)
	}

	err = amqp.BindQueueToExchange(ebCh, c.EventBusQueue, c.EventBusExchange, "warehouse.catalog.stockUpdated")
	if err != nil {
		logger.Log("Failed to prepare EventBus queue: %v\n", err)
		os.Exit(1)
	}

	// event consume
	msgs, err := ebCh.Consume(
		c.EventBusQueue, // queue
		"",              // consumer
		false,           // auto-ack
		false,           // exclusive
		false,           // no-local
		false,           // no-wait
		nil,             // args
	)
	if err != nil {
		logger.Log("Failed to register a consumer: %s", err)
		os.Exit(1)
	}

	var forever chan struct{}
	go func() {
		for d := range msgs {
			msg, err := amqp.Deserialize(d.Body)
			if err != nil {
				logger.Log("json error: %v\n", err)
				d.Reject(false) // FIXME: how to handle erros in queue...????
				continue
			}
			event := fmt.Sprintf("%s", msg["event"])
			data := (msg["data"]).(map[string]interface{})
			logger.Log("Message<%s>: %s\n", event, data)

			// switch true {
			// case strings.Contains(event, amqp.EVENT_PRODUCT_ADD):
			// 	// create new basket and add product to it
			// 	logger.Log("Event: %s", amqp.EVENT_PRODUCT_ADD)
			// case strings.Contains(event, amqp.EVENT_PRODUCT_REMOVE):
			// 	// remove product from basket
			// 	logger.Log("Event: %s", amqp.EVENT_PRODUCT_REMOVE)
			// }

			logger.Log("ACK: %s", event)
			d.Ack(false)
		}
	}()

	logger.Log("Waiting for messages...")
	<-forever
}
